#!/bin/sh
# the next line restarts using wish \
exec wish "$0" "$@"

###############################################################
# Minihelp - Help Editor
# Charlie KEMPSON - charlie@siren.demon.co.uk
# http://public.logica.com/~kempsonc
###############################################################

###############################################################
#
#    Copyright (c) 1995-1996 Charlie Kempson
#
#    This program is free software; you can redistribute it 
#    and/or modify it under the terms of the GNU General 
#    Public License as published by the Free Software 
#    Foundation (version 2 of the License).
#
#    This program is distributed in the hope that it will 
#    be useful, but WITHOUT ANY WARRANTY; without even the 
#    implied warranty of MERCHANTABILITY or FITNESS FOR A 
#    PARTICULAR PURPOSE.  See the GNU General Public License 
#    for more details.
#
#    For a copy of the GNU General Public License, write to the 
#    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
#    MA 02139, USA.
###############################################################

###############################################################
# Set include path
set GLOBAL_INCLUDE [file dirname [info script]]

###############################################################
# The procedure to read other tcl files stored in
# /usr/local/lib/TkNet/modules
proc MH_SourceOther { filename } {

   # Globals
   global GLOBAL_INCLUDE

   # Look in the global area for files
   if [file exists "$GLOBAL_INCLUDE/$filename"] {
      if [catch {uplevel #0 source "$GLOBAL_INCLUDE/$filename"}] {
         Info_Dialog . "Error reading file $GLOBAL_INCLUDE/$filename: this doesn't look like a Tcl archive"
      }
   } else {
      # FILE MISSING
      Info_Dialog . "File $filename missing from $GLOBAL_INCLUDE"
   }
}

###############################################################
# The procedure for popping up a popup menu
proc MH_PopupMenu { parent window } {

   # Get current mouse position
   set x [ winfo pointerx $parent ]
   set y [ winfo pointery $parent ]

   # Popup the menu
   tk_popup $window $x $y
}

###############################################################
# Change cursor to an hourglass and back again
proc MH_WatchCursor {} {

   # Loop through children setting the cursor
        set busy ""
        foreach w [winfo children .] {
                lappend busy [list $w [lindex [$w config -cursor] 4]]
        }
        foreach w $busy {catch {[lindex $w 0] config -cursor watch}}
        update idletasks
}
proc MH_NormalCursor {} {

   # Loop through children setting the cursor
        set busy ""
        foreach w [winfo children .] {
                lappend notbusy [list $w [lindex [$w config -cursor] 4]]
        }
        foreach w $notbusy {catch {[lindex $w 0] config -cursor hand2}}
        update idletasks
}

###############################################################
# Procedure to open a help file
proc MH_OpenProject { } {

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING TEXT_COLOUR \
      gb_current_page_changed gb_page_autocommit \
      gs_current_project

   # Check whether page has changed
   if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
      MH_SaveHelpPage
   }
   if {$gb_current_page_changed != 0} {
      if [Question_Dialog . "Help text on this page has changed!" \
         "Discard Changes" "Cancel"] {
         return
      }
   }

   # Prompt for the project to open
   set gs_current_project [File_Select "Open Project"]
   if {$gs_current_project != ""} {MH_ReadProject}
}

###############################################################
# Procedure to import a page from a file
proc MH_ImportFile {{import_file ""}} {

   # Globals
   global gt_edit_help

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Prompt for a file to import
   if {$import_file == ""} {
      set import_file [File_Select "Import File"]
      if {$import_file == ""} {return}
   }
   if ![file readable $import_file] {
      Info_Dialog . "Unable to read file $import_file:\nfile does not exist or is not readable"
      return
   }

   # Read file
   set fd [open $import_file r]
   if [catch {set text [read $fd]} error] {
      Info_Dialog . "Unable to read file $import_file:\n$error"
      return
   }
   close $fd

   # Ask the user where they wish to import to...
   if [Question_Dialog . "Insert into Clipboard or at Insertion Cursor?" \
      "At Cursor" "Into Clipboard"] {
      clipboard clear
      clipboard append $text
   } else {
      # Insert text at current position
      $gt_edit_help insert insert $text
      .name_fr.save configure -state normal
      set gb_current_page_changed 1
   }
}

###############################################################
# Procedure to import an HTML file
proc MH_ImportHTML {{import_file ""}} {

   # Globals
   global gt_edit_help

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Prompt for a file to import
   if {$import_file == ""} {
      set import_file [File_Select "Import File"]
      if {$import_file == ""} {return}
   }
   if ![file readable $import_file] {
      Info_Dialog . "Unable to read file $import_file:\nfile does not exist or is not readable"
      return
   }

   # Insert text at current position
   MH_ReadSGMLFile $import_file $gt_edit_help 
}

###############################################################
# Procedure to open a help file
proc MH_ReadProject { } {

   # Globals
   global gs_current_project gas_history \
      gt_edit_help gb_current_page_changed \
      gs_image_base_dir gs_sgml_structure

   # Blank old information
   $gt_edit_help delete 1.0 end
   .name_fr.name configure -state normal
   .name_fr.name delete 0 end
   .name_fr.name configure -state disabled
   set gs_image_base_dir ""
   MH_EnableEdit

   if {[MH_ReadSGMLFile $gs_current_project $gt_edit_help] == 0} {

      # Save the project name
      global $gs_sgml_structure
      set title [subst \$${gs_sgml_structure}(Title)]
      wm title . "Edit Help - $title"

      # Add to the windows menu
      .mbar.windows.menu add command -label $gs_current_project \
         -command "MH_SwitchToProject $gs_sgml_structure"
      
      # Check the image base
      if {$gs_image_base_dir == ""} {
         set gs_image_base_dir [file dirname $gs_current_project ]
      }

      # Display the page selector
      MH_SelectEditHelpPage . FULL
 
   } else {
      # Load failed
      MH_DisableEdit
   }
}

###############################################################
# Procedure to open a help file
proc MH_SwitchToProject { project {display_select 1} } {

   # Globals
   global gs_sgml_structure gt_edit_help gb_current_page_changed \
      gb_page_autocommit gs_current_project gs_image_base_dir

   # Check whether page has changed
   if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
      MH_SaveHelpPage
   }
   if {$gb_current_page_changed != 0} {
      if [Question_Dialog . "Help text on this page has changed!" \
         "Discard Changes" "Cancel"] {
         return
      }
   }

   # Clear the current page
   $gt_edit_help delete 1.0 end

   # Save the project name
   set gs_sgml_structure $project
   global $gs_sgml_structure 
   set title [subst \$${gs_sgml_structure}(Title)]
   set gs_image_base_dir [subst \$${gs_sgml_structure}(Path)]
   set gs_current_project [subst \$${gs_sgml_structure}(FullFileName)]
   wm title . "Edit Help - $title"

   # Display the page selector
   if {$display_select == 1} {MH_SelectEditHelpPage . FULL}
}

###############################################################
# Procedure to save a help file
proc MH_SaveProject { {name ""} } {

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Globals
   global gb_page_autocommit gb_current_page_changed \
      gs_current_project gs_image_base_dir \
      gs_sgml_structure

   # Check the name for syntactic correctness
   if {[llength [split $name]] > 1 || [regexp -nocase \
      {[`|!$%^&*()|~<>,]+} $name]} {
      Info_Dialog . "Chosen name is not valid.
The project has not been saved."
      return
   }

   # If project does not have a name pop
   # up the save project as dialog.
   if {$name == ""} {
      MH_SaveProjectAs
      return
   }

   # If the current page has been changed, ask the user if
   # (s)he wishes to save the changes to the current page
   # before saving.
   if {$gb_current_page_changed == 1} {
      if {$gb_page_autocommit == 1 || [Question_Dialog . \
         "The current page has changed.  Do you wish
 to save these changes with the project?" "No" "Yes"]} {
         MH_SaveHelpPage
      }
   }

   # Now write the help text itself - get the project root
   regsub {[`"#;:',.?/\|!$%^&*()|~<>,-]+} [file tail [file \
      rootname $gs_current_project]] {} filename
   # Check whether the data exists

   # Open the file
   if [catch {set fp [open $name w]}] {
      Info_Dialog . "Unable to open file $name for writing"
      return
   }

   # Write the header
   puts $fp {<!DOCTYPE minihelp11 [

   <!-- - - - - - - - - - - - - - - -  -->
   <!-- DTD for MiniHelp helptext pages -->
   <!-- - - - - - - - - - - - - - - -  -->

   <!-- Tag types and rules -->
   <!ELEMENT minihelp11 - - (page+)
      -- One or more pages must be present 
         May have a title and a colon separated image path -->
   <!ATTLIST minihelp11  title       CDATA    #IMPLIED 
                         imgpath     CDATA    #IMPLIED > 

   <!ELEMENT page     - - (title, p*)
      -- Page must have a title and a number of paragraphs
         Plus attributes for the page level and ID -->
   <!ATTLIST page      lvl         NUMBER   1 
                       ID          ID       #IMPLIED >

   <!ELEMENT title    - - (#PCDATA)
      -- Title comprises data only -->
   <!ELEMENT b        - - ((#PCDATA | p | br | ul | xref | img | center)*)
      -- Bold may be underlined or contain other tags -->
   <!ELEMENT i        - - ((#PCDATA | p | br | ul | xref | img | center)*)
      -- Italic may be underlined or contain other tags -->
   <!ELEMENT ul       - - ((#PCDATA | p | br | b | i | tt | xref | img | center)*)
      -- Underline may contain other tags -->
   <!ELEMENT tt       - - ((#PCDATA | p | br | ul | xref | img | center)*)
      -- Type may be underlined or contain other tags -->
   <!ELEMENT center   - - ((#PCDATA | p | br | ul | b | tt | i | xref | img)*)
      -- Formatting may contain other entities -->

   <!ELEMENT br       - o EMPTY
      -- Break comprises empty data -->
   <!ELEMENT p        - o ((#PCDATA | br | b | i | ul | tt | xref | img | center)*)
      -- Paragraph comprises data plus other tags 
         Plus attribute for paragraph alignment -->
   <!ATTLIST p         align       (CENTER,LEFT,RIGHT)   LEFT > 

   <!ELEMENT xref     - - ((#PCDATA | br | ul | b | tt | i | img | center)*)
      -- Hyperlink may be underlined or contain images 
         Plus attribute linking to unique page ID -->
   <!ATTLIST xref      page        IDREF    #IMPLIED >
 
   <!ELEMENT img      - O EMPTY
      -- Images may not contain any other data 
         But may have a source and an alignment -->
   <!ATTLIST img       src         CDATA                 #IMPLIED 
                       align       (CENTER,LEFT,RIGHT)   LEFT 
                       alt         CDATA                 #IMPLIED> 

   <!-- Special character support -->
   <!ENTITY  amper     SDATA   "&" > 
   <!ENTITY  lt        SDATA   "<" > 
   <!ENTITY  gt        SDATA   ">" > 
   <!ENTITY  dollar    SDATA   "$" >
   <!ENTITY  lbrace    SDATA   "{" >
   <!ENTITY  rbrace    SDATA   "}" >
   <!ENTITY  lsquare   SDATA   "[" >
   <!ENTITY  rsquare   SDATA   "]" >
   <!ENTITY  quot      SDATA   '"' >
   <!ENTITY  squot     SDATA   "'" >
   <!ENTITY  backslash SDATA   "/" >
]>
}

   global $gs_sgml_structure

   # Create the document element + attributes
   set header "<MINIHELP11"
   set title [subst \$${gs_sgml_structure}(Title)]
   if {![string compare $title ""] || ![string compare $title "No Title"]} {
      Info_Dialog . "WARNING: Help file saved without a title!"
   } else {append header " TITLE=\"$title\""}
   set imgpath [subst \$${gs_sgml_structure}(Path)]
   if [string compare $title ""] {append header " IMGPATH=\"$imgpath\""}
   puts $fp "$header>\n\n"

   # Create each page in the order specified by the TOC
   set as_list [subst \$${gs_sgml_structure}(TOC)]
   foreach page $as_list {
      set local_text [subst \$${gs_sgml_structure}(${page}_Text)]
      puts $fp "$local_text\n"
   }
   puts $fp "</MINIHELP11>"

   # Flush the file
   flush $fp
   close $fp

   # Inform the user
   Info_Dialog . "Help saved to file $name"
   set ${gs_sgml_structure}(Modified) 0
   set gs_current_project $name
   wm title . "Edit Help - $title"
}

###############################################################
# Procedure to save a help file
proc MH_SaveProjectAs { } {

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING gs_new_project_name \
      TEXT_COLOUR

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Prompt for the project to save as
   set gs_new_project_name [File_Select "Save Project As..."]
   if [file exists $gs_new_project_name] {
      if [Question_Dialog . "Are you sure you wish to overwrite
file $gs_new_project_name ?" "Yes" "No"] {
         Info_Dialog . "Help not saved"
         return
      }
   }

   # Check validity of name
   if {$gs_new_project_name != ""} {
      MH_SaveProject $gs_new_project_name
   } else {
      Info_Dialog . "Help archive not saved"
   }
}

###############################################################
# Procedure to close a help file.  Returns 1 if sucessful, -1
# if not successful and 0 if no more projects exist.
proc MH_CloseProject { {display_select 1} } {

   # Globals
   global gb_current_page_changed gs_current_project \
      gs_new_project_name gs_MH_NewHelpPage \
      gt_edit_help gs_sgml_structure

   # Check for current project
   # Check for project
   if ![MH_IsCurrentProject] {return 0}

   # Check for changes to the project
   global $gs_sgml_structure
   if {$gb_current_page_changed || [subst \$${gs_sgml_structure}(Modified)]} {
      if [Question_Dialog . "The project $gs_current_project has changed.
Do you wish to discard changes?" "Discard" "Cancel"] {
         # User requested cancel
         return -1
      }
   }

   # Find the entry in the windows menu and delete
   set bFound 0
   for {set icount 0} {$bFound == 0} {incr icount} {
      set filename [.mbar.windows.menu entrycget $icount -label]
      #Info_Dialog . "$filename (looking for ${gs_current_project})"
      if ![string compare $filename $gs_current_project] {
         .mbar.windows.menu delete $icount
         set bFound 1
      }
   }

   if {[info globals $gs_sgml_structure] != ""} {
      unset $gs_sgml_structure
   }

   # Set to next available project (if there is one)
   set filename [.mbar.windows.menu entrycget 0 -label]
   #Info_Dialog . $filename
   if {$filename != ""} {
      regsub {[`"#;:',.?/\|!$%^&*()|~<>,-]+} [file tail [file \
         rootname $filename]] {} filename
      set gs_sgml_structure gsSGML_${filename}
      MH_SwitchToProject $gs_sgml_structure $display_select
   } else {
      wm title . "Edit Help - <no project>"
      set gs_current_project      ""
      set gs_new_project_name     ""
      set gs_sgml_structure       ""
   }
   set gs_MH_NewHelpPage       ""
   set gb_current_page_changed 0

   # Reset all user fields
   .name_fr.name configure -state normal
   .name_fr.name delete 0 end
   .name_fr.name configure -state disabled
   $gt_edit_help delete 1.0 end
   MH_DisableEdit

   # Completed successfully
   return 1
}

###############################################################
# Procedure to exit minihelp
proc MH_Exit { } {

   # Check for current project
   # Check for project
   if ![MH_IsCurrentProject] {exit}

   set status [MH_CloseProject 0]
   while {$status == 1} {set status [MH_CloseProject 0]}
   if {$status == 0} {exit} else {return}
}

###############################################################
# Procedure for adding a title to a file
proc MH_AddTitle {} {

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING \
      TEXT_COLOUR gs_sgml_structure
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Popup a selection window
   set window .title_page
   if [winfo exists $window] {
       # Pop it up!
       wm deiconify $window
       raise $window
       return
   }

   # Otherwise create the window
   toplevel $window
   wm title $window "Add Title to File"
   wm transient $window .

   # Create a frame containing a list and an entry field
   frame $window.sel_fr -borderwidth $RIDGE_BORDER -relief groove
   pack $window.sel_fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING \
      -side top -fill x
   label $window.sel_fr.label -text "Title String"
   entry $window.sel_fr.entry -width 30 -bg $TEXT_COLOUR
   pack $window.sel_fr.label $window.sel_fr.entry -side left \
      -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING
   $window.sel_fr.entry insert end [subst \$${gs_sgml_structure}(Title)]

   # And add buttons to the window
   set frame [frame $window.button_fr]
   pack $frame -side bottom -fill x
   button $frame.close -text Cancel -command "destroy $window"
   button $frame.ok -text OK -command {
      global gs_sgml_structure
      global $gs_sgml_structure
      set title [.title_page.sel_fr.entry get]
      set ${gs_sgml_structure}(Title) $title
      wm title . "Edit Help - $title"
      destroy .title_page
   }
   bind $window.sel_fr.entry <Return> "$frame.ok invoke"
   pack $frame.ok $frame.close -side right -padx $DEFAULT_PADDING -pady \
      $DEFAULT_PADDING

   # And now centre it on its parent
   Centre_Dialog $window widget .
   focus $window.sel_fr.entry
}

###############################################################
# The procedure to change the base directory for images
proc MH_ImageChangeBase { } {

   # Global
   global gs_image_base_dir gs_current_project gs_sgml_structure
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # If the current image base directory is "" then
   # default to the save directory
   if {$gs_image_base_dir == ""} {
      if {$gs_current_project != ""} {
         # Default to save directory
         set gs_image_base_dir [file dirname $gs_current_project]
      } else {
         # Default to current directory
         set gs_image_base_dir [pwd]
      }
   }

   # Popup a file selection dialogue in directory mode
   # to prompt for selection
   set gs_image_base_dir [File_Select "Select Image Directory" \
      "*" $gs_image_base_dir 1]
   set gs_image_base_dir [string trimright $gs_image_base_dir "/\\"]
   set ${gs_sgml_structure}(Path) $gs_image_base_dir
   # Info_Dialog . $gs_image_base_dir
}

###############################################################
# The procedure to rebind an image
proc MH_BindImage {} {

   # Globals
}

###############################################################
# The procedure to navigate backwards through the history
proc MH_SelectPageFromList { type } {

   # Globals
   global gas_history gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

	set page [.select_help.list_fr.list curselection]
	if {$page == ""} {
		return
	} else {

		# Type may be HISTORY or FULL
		if {$type == "HISTORY"} {
			set as_list $gas_history
      } else {
         global $gs_sgml_structure
         set as_list [subst \$${gs_sgml_structure}(TOC)]
      }

      set name [lindex $as_list $page]
      if {$type == "HISTORY"} {set gas_history \
         [lreplace $gas_history $page end]}
      MH_DisplayHelpPage $name
      after 400 destroy .select_help
   }
}

###############################################################
# The procedure to navigate backwards through the history
proc MH_EditNavigateBack {} {

   # Globals
   global gas_history

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # The list gas_history contains a list of all help pages
   # visited.  Simply extract the second to last page, and
   # delete the last item in the list, then dislpay the 
   # page.
   # Info_Dialog . $gas_history
   if {[llength $gas_history] < 2} {
      # Nowhere to return to
      return
   }
   
   # Get the last page
   set help [lindex $gas_history [expr [llength $gas_history] - 2]]

   # Delete the two last items (note that one of them is recreated
   # when we call MH_DisplayHelpPage).
   set gas_history [lreplace $gas_history [expr [llength $gas_history] \
      - 2] end]

   # And show the help page
   MH_DisplayHelpPage $help
}

###############################################################
# Procedure for displaying a help page
proc MH_DisplayHelpPage { text } {

   # Globals
   global gt_edit_help gb_current_page_changed \
      gas_history gs_image_base_dir gs_sgml_structure \
      gb_show_formatted

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Check for the existence of the page before going
   # ahead...
   global $gs_sgml_structure
   set list [array names $gs_sgml_structure]
   if {[lsearch $list "${text}_Text"] == -1} {
      Info_Dialog . "Page does not exist!"
      return
   }

   # Display the associated help text
   MH_EnableEdit
   $gt_edit_help delete 1.0 end

   # Parse the help (0=Edit rather than run time)
   if {$gb_show_formatted == 1} {
      MH_MarkupToText $gt_edit_help INSERT 0 $gs_image_base_dir \
         [subst \$${gs_sgml_structure}(${text}_Text)]
   } else {
      $gt_edit_help insert end [subst \$${gs_sgml_structure}(${text}_Text)]
   }
   set gb_current_page_changed 0 

   # Set the focus
   focus $gt_edit_help

   # Display the page title in the entry
   .name_fr.name configure -state normal
   .name_fr.name delete 0 end
   .name_fr.name insert end $text
   .name_fr.name configure -state disabled
   .name_fr.save configure -state disabled

   # Add to the history list (removing any other references to
   # the page on the way - there will be at most one given the
   # nature of the check)
   set item [lsearch -exact $gas_history $text]
   if {$item != -1} {set gas_history [lreplace $gas_history $item $item]}
   lappend gas_history $text
}

###############################################################
# Procedure for displaying a new help page
proc MH_NewHelpPage {{location "end"} {level "1"}} {

   # If the variable keep_old_text is set, then 
   # the text that is currently being displayed
   # is retained.

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING gs_MH_NewHelpPage \
      gt_edit_help TEXT_COLOUR gb_keep_old_text \
      gas_history gs_sgml_structure
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Popup a selection window
   set window .new_page
   if [winfo exists $window] {
       # Pop it up!
       wm deiconify $window
       raise $window
       return
   }

   # Otherwise create the window
   toplevel $window
   wm title $window "New Help Page"
   wm transient $window .

   # Create a frame containing a list and an entry field
   frame $window.sel_fr -borderwidth $RIDGE_BORDER -relief groove
   pack $window.sel_fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING \
      -side top -fill x
   label $window.sel_fr.label -text "Page Name"
   entry $window.sel_fr.entry -width 30 -bg $TEXT_COLOUR -textvariable \
      gs_MH_NewHelpPage
   pack $window.sel_fr.label $window.sel_fr.entry -side left \
      -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING

   # And add buttons to the window
   set frame [frame $window.button_fr]
   pack $frame -side bottom -fill x
   button $frame.close -text Cancel -command "wm withdraw $window"
   button $frame.ok -text OK -command "MH_CreateNewHelpPage $location $level"
   bind $window.sel_fr.entry <Return> "$frame.ok invoke"
   pack $frame.ok $frame.close -side right -padx $DEFAULT_PADDING -pady \
      $DEFAULT_PADDING

   # And now centre it on its parent
   Centre_Dialog $window widget .
   focus $window.sel_fr.entry
}

###############################################################
# Procedure for creating a new help page
proc MH_CreateNewHelpPage {{location "end"} {level "1"}} {

   # If the variable keep_old_text is set, then 
   # the text that is currently being displayed
   # is retained.

   # Globals
   global gs_MH_NewHelpPage gt_edit_help gb_keep_old_text \
      gas_history gs_sgml_structure
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Check the name for syntactic correctness
   if {[llength [split $gs_MH_NewHelpPage]] > 1 || [regexp -nocase \
      {[`"#;:',.?/\|!$%^&*()|~<>,-]+} $gs_MH_NewHelpPage] ||
      $gs_MH_NewHelpPage == ""} {
      Info_Dialog . {Chosen name is not valid.  Please input 
another name.  It must contain only alphanumeric
characters.}
   } else {

      # Create a new global
      lappend gas_history $gs_MH_NewHelpPage
      set ${gs_sgml_structure}(${gs_MH_NewHelpPage}_Text) {}
      set ${gs_sgml_structure}(${gs_MH_NewHelpPage}_Level) $level
      set ${gs_sgml_structure}(TOC) [linsert \
         [subst \$${gs_sgml_structure}(TOC)] $location $gs_MH_NewHelpPage]

      # Clear the text area
      MH_EnableEdit
      if {$gb_keep_old_text == 0} {
         $gt_edit_help delete 1.0 end
      } else {
         # Reset flag
         set gb_keep_old_text 0
      }

      # Display the page title in the entry
      .name_fr.name configure -state normal
      .name_fr.name delete 0 end
      .name_fr.name insert end $gs_MH_NewHelpPage
      .name_fr.name configure -state disabled

      # Set the focus
      focus $gt_edit_help

      # Page should be saved (project has changed)
      .name_fr.save configure -state normal
      set gb_current_page_changed 1

      # Withdraw this window
      wm withdraw .new_page
   }
}

###############################################################
# The procedure for finding tags in the help
proc MH_ShowHyperMenu { tag } {

   # Globals
   global gt_edit_help

   # Create the menu - Check for existence
   set window .hyper_popup_menu
   if [winfo exists $window] {destroy $window}

   # Find the start and end of the current tag
   set x [ winfo pointerx $gt_edit_help ]
   set y [ winfo pointery $gt_edit_help ]
   set curpos [$gt_edit_help index @$x,$y]
   set ranges [$gt_edit_help tag ranges $tag]
   set list_len [llength $ranges]
   for {set count 0} {$count < $list_len} {incr count 2} {
      if {$curpos >= [lindex $ranges $count]} {
         set start [lindex $ranges $count]
         set end [lindex $ranges [expr $count + 1]]
         #Info_Dialog . "Tag found $start $end"
      }
   }

   # Create the popup menu
   menu .hyper_popup_menu -tearoff 0
      $window add checkbutton -label $tag -indicatoron False
      $window add separator
      $window add command -label "Open Link" -command "MH_OpenLink \
         $tag" -underline 0
      $window add command -label "Bind Link" -command "MH_BindLink \
         $tag $start $end" -underline 0

   # Now popup the window in place
   MH_PopupMenu . $window
}

###############################################################
# The procedure for finding tags in the help
proc MH_BindLink { tag start end } {

   # Globals
   global RIDGE_BORDER gs_sgml_structure \
      DEFAULT_PADDING TEXT_COLOUR gt_edit_help
   global $gs_sgml_structure
   set as_list [subst \$${gs_sgml_structure}(TOC)]

   # Test for NULL arguments.  If so, use current
   # selection...
   if {$start == ""} {
      # Info_Dialog . "Using current selection"
      if [catch {$gt_edit_help get sel.first}] {return}
      set tagstart [$gt_edit_help index sel.first]
      set tagend [$gt_edit_help index sel.last]

      # Ensure that the current selection has not
      # already got a hyperlink tag
      MH_RemoveTags XREF
   }
   if {$tag == ""} {set tag NONE}

   # Test for window
   # Popup a selection window
   set window .bind_hyperlink
   if [winfo exists $window] {destroy $window}
   toplevel $window
   wm title $window "Select Hyperlink"
   wm transient $window .

   # Create a frame containing a list
   frame $window.sel_fr -borderwidth $RIDGE_BORDER -relief groove
   pack $window.sel_fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING \
      -side top -fill x
   set list [ScrolledList $window.list_fr 0 20 browse 0]
   $list configure -bg $TEXT_COLOUR
   pack forget $window.list_fr
   pack $window.list_fr -side top -fill both -padx $DEFAULT_PADDING \
      -pady $DEFAULT_PADDING -expand true

   # Add the help pages title to the list
   foreach item $as_list {
      # Get the help text title for the page
      set text "Unknown Help Page"
      global $item
      set local_text [subst \$${gs_sgml_structure}(${item}_Text)]
      set start [string first "<title>" $local_text]
      if {$start == -1} {set start [string first "<TITLE>" $local_text]}
      set end [string first "</title>" $local_text]
      if {$end == -1} {set end [string first "</TITLE>" $local_text]}

      if {$start != -1} {
         set start [expr [string wordend $local_text [expr $start + 1]] +1]
         set end [expr [string wordstart $local_text $end] -1]
         set text [string range $local_text $start $end]
         set text [string trimleft $text]
      } else {
         Info_Dialog . "Error in help text (${item})!  
All pages must begin with a title e.g. 
<title>Contents</title> or
<TITLE>Contents</TITLE>"
      }

      # Add indentation for the levels
      set litem ""
      for {set count 1} {$count < [subst \
         \$${gs_sgml_structure}(${item}_Level)]} {incr count} {
         append litem "\t"
      }
      append litem $text
      $list insert end $text
   }

   # Now for a goto, close and contents button
   set frame [frame $window.button_fr]
   pack $frame -side bottom -fill x
   button $frame.cancel -text Cancel -command "destroy $window"
   button $frame.bind -text OK -command "MH_CreateLink $tag $tagstart $tagend"
   pack $frame.cancel $frame.bind -side right \
      -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING

   # And now centre it on its parent
   Centre_Dialog $window widget .
}

###############################################################
# The procedure for opening a tag in the help
proc MH_CreateLink { tag start end } {

   # Globals
   global gt_edit_help RED GREEN FONT_BOLD \
      gb_current_page_changed gb_project_changed \
      gs_sgml_structure
   global $gs_sgml_structure

   set page [.bind_hyperlink.list_fr.list curselection]
   if {$page == ""} {
      return
   } else {
      set name [lindex [subst \$${gs_sgml_structure}(TOC)] $page]
      if {$tag != "NONE"} {
         $gt_edit_help tag remove $tag $start $end
      }
      $gt_edit_help tag add $name $start $end
      $gt_edit_help tag configure $name -foreground \
         $GREEN -font $FONT_BOLD
      $gt_edit_help tag bind $name <Enter> \
         "$gt_edit_help tag configure $name -foreground $RED"
      $gt_edit_help tag bind $name <Leave> \
         "$gt_edit_help tag configure $name -foreground $GREEN"
      $gt_edit_help tag bind $name <ButtonPress> "MH_ShowHyperMenu $name"

      # Page has changed
      set gb_project_changed      1
      set gb_current_page_changed 1
      .name_fr.save configure -state normal

      # Destroy the popup
      destroy .bind_hyperlink
   }
}

###############################################################
# The procedure for opening a tag in the help
proc MH_OpenLink { tag } {

   # Globals
   global gb_current_page_changed gb_page_autocommit

   # Check for changed
   if {$gb_current_page_changed == 0 || $gb_page_autocommit == 1} {
      MH_SaveHelpPage
      MH_DisplayHelpPage $tag
   } else {
      if ![Question_Dialog . "Help text on this page has changed!" \
         "Discard Changes" "Cancel"] {
         MH_DisplayHelpPage $tag
      }
   }
}

###############################################################
# The procedure to let you select a page
proc MH_SelectEditHelpPage { parent type } {

   # Type may be HISTORY or FULL

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING TEXT_COLOUR \
      gas_history gs_sgml_structure gs_indent_string

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Test for window
   # Popup a selection window
   set window .select_help
   if [winfo exists $window] {
       # Pop it up!
       wm deiconify $window
       raise $window
       update
       return
   }

   toplevel $window
   if {$type == "HISTORY"} {
      wm title $window "History"
      set as_list $gas_history
	} else {
		wm title $window "Select Page"
      global $gs_sgml_structure
		set as_list [subst \$${gs_sgml_structure}(TOC)]
	}
	wm transient $window .

	# Create a frame containing a list
	frame $window.sel_fr -borderwidth $RIDGE_BORDER -relief groove
	pack $window.sel_fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING \
		-side top -fill x
	set list [ScrolledList $window.sel_fr.list_fr 0 20 browse 0]
	$list configure -bg $TEXT_COLOUR

   # Create a frame containing the TOC contruction buttons
   set frame [frame $window.sel_fr.button_fr]
   if {$type != "HISTORY"} {
      button $frame.up -width 10 -text "Move Up" -command "MH_MovePage $window $list UP"
      button $frame.down -width 10 -text "Move Down" -command "MH_MovePage $window $list DOWN"
      button $frame.indent -width 10 -text "-->" -command "MH_IndentPage $window $list IN"
      button $frame.dedent -width 10 -text "<--" -command "MH_IndentPage $window $list OUT"
      label $frame.blank -width 10 -text ""
      pack $frame.up $frame.indent $frame.dedent $frame.down $frame.blank -side top
   }
	button $frame.close -width 10 -text Close -command "destroy $window"
	button $frame.new -width 10 -text New -command "destroy $window; MH_NewHelpPage"
	button $frame.goto -width 10 -text Goto -command "MH_GotoSelectedHelpPage $window $list $type"
   pack $frame.new $frame.goto $frame.close -side top

   # Repack the two frames
	pack forget $window.sel_fr.list_fr
	pack $window.sel_fr.button_fr -side left -fill y -padx $DEFAULT_PADDING \
		-pady $DEFAULT_PADDING -expand true
	pack $window.sel_fr.list_fr -side left -fill both -padx $DEFAULT_PADDING \
		-pady $DEFAULT_PADDING -expand true

   # Add the help pages title to the list
   foreach item $as_list {
      # Get the help text title for the page
      set text "Unknown Help Page"
      set local_text [subst \$${gs_sgml_structure}(${item}_Text)]
      set start [string first "<title>" $local_text]
      if {$start == -1} {set start [string first "<TITLE>" $local_text]}
      set end [string first "</title>" $local_text]
      if {$end == -1} {set end [string first "</TITLE>" $local_text]}

      if {$start != -1} {
         set start [expr [string wordend $local_text [expr $start + 1]] +1]
         set end [expr [string wordstart $local_text $end] -1]
         set text [string range $local_text $start $end]
         set text [string trimleft $text]
      } else {
         Info_Dialog . "Error in help text (${item})!  
All pages must begin with a title e.g. 
<title>Contents</title> or
<TITLE>Contents</TITLE>"
      }

      # Add indentation for the levels
      # Info_Dialog . "$text : [subst \$${gs_sgml_structure}(${item}_Level)]"
      set litem ""
      for {set count 1} {$count < [subst \
         \$${gs_sgml_structure}(${item}_Level)]} {incr count} {
         append litem $gs_indent_string
      }
      append litem $text
      $list insert end $litem
   }
	bind $list <Double-ButtonRelease-1> "MH_GotoSelectedHelpPage $window $list $type"

	# And now centre it on its parent
	Centre_Dialog $window widget $parent
}

###############################################################
# The procedure to goto a specified page from the history
proc MH_GotoSelectedHelpPage { window list type } {

   # Globals
   global gas_history gs_sgml_structure
   global $gs_sgml_structure
   set as_list [subst \$${gs_sgml_structure}(TOC)]

   set selected [$list curselection]
   if {$selected == ""} {return}
   set text [lindex $as_list $selected]

   # Show the new page
   MH_DisplayHelpPage $text
   after 400 "destroy $window"
}

###############################################################
# The procedure to move a page within the document order
proc MH_MovePage { window list type } {

   # Type may be UP or DOWN

   # Globals
   global gs_sgml_structure gb_project_changed
   global $gs_sgml_structure
   set as_list [subst \$${gs_sgml_structure}(TOC)]

   set selected [$list curselection]
   if {$selected == ""} {return}

   set text [lindex $as_list $selected]
   set listtext [$list get $selected]
   set as_list [lreplace $as_list $selected $selected]
   $list delete $selected
   if {$type == "UP"} {
      set as_list [linsert $as_list [expr $selected -1] $text]
      $list insert [expr $selected -1] $listtext
      $list selection set [expr $selected -1]
   } else {
      set as_list [linsert $as_list [expr $selected +1] $text]
      $list insert [expr $selected +1] $listtext
      $list selection set [expr $selected +1]
   }
   set ${gs_sgml_structure}(TOC) $as_list
   set gb_project_changed 1
}

###############################################################
# The procedure to indent a page within the document order
proc MH_IndentPage { window list type } {

   # Type may be IN or OUT

   # Globals
   global gs_sgml_structure gb_project_changed \
      gs_indent_string
   global $gs_sgml_structure
   set as_list [subst \$${gs_sgml_structure}(TOC)]

   set selected [$list curselection]
   if {$selected == ""} {return}

   set text [string trim [$list get $selected]]
   set item [lindex $as_list $selected]
   $list delete $selected
   if {$type == "IN"} {
      incr ${gs_sgml_structure}(${item}_Level)
   } else {
      incr ${gs_sgml_structure}(${item}_Level) -1
      if {[subst \$${gs_sgml_structure}(${item}_Level)] < 1} {
         set ${gs_sgml_structure}(${item}_Level) 1
      }
   }

   # Set the level into the saved page
   set pagetext [subst \$${gs_sgml_structure}(${item}_Text)]
   set ${gs_sgml_structure}(${item}_Text) "<PAGE ID=\""
   append ${gs_sgml_structure}(${item}_Text) $item "\" LVL=\"" \
      [subst \$${gs_sgml_structure}(${item}_Level)] "\">" \
      [string range $pagetext [expr [string first {>} $pagetext] \
      +1] end]

   set litem ""
   for {set count 1} {$count < [subst \
      \$${gs_sgml_structure}(${item}_Level)]} {incr count} {
      append litem $gs_indent_string
   }
   append litem $text
   $list insert $selected $litem
   $list selection set $selected
   set gb_project_changed 1
}

###############################################################
# The procedure for finding tags in the help
proc MH_RenamePage { } {

   # Global
   global gb_keep_old_text gas_history gs_sgml_structure
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Popup a warning dialogue
   Info_Dialog . "All references to this page must be relinked!"

   # All we need do is to call MH_NewHelpPage, asking the
   # routine to retain the current information.  Then
   # we delete the old variable reference.
   set gb_keep_old_text 1
   set name [.name_fr.name get]
   unset ${gs_sgml_structure}(${name}_Text)
   set level [subst \$${gs_sgml_structure}(${name}_Level)]
   unset ${gs_sgml_structure}(${name}_Level)

   # Delete from history, and delete from list of pages
   set location [lsearch -exact [subst \$${gs_sgml_structure}(TOC)] $name]
   if {$location != -1} {
      set ${gs_sgml_structure}(TOC) [lreplace \
         [subst \$${gs_sgml_structure}(TOC)] $location $location]
   }
   set hist [lsearch -exact $gas_history $name]
   if {$hist != -1} {
      set gas_history [lreplace $gas_history $hist $hist]
   }

   # Create the new page
   MH_NewHelpPage $location $level
   # Fill the field with default information, and
   # disable the cancel button
   wm title .new_page "Rename Page"
   .new_page.sel_fr.entry delete 0 end
   .new_page.sel_fr.entry insert end [.name_fr.name get]
   pack forget .new_page.button_fr.close
}

###############################################################
# The procedure for starting the run-time system from 
# the current page
proc MH_PreviewPage { } {

   # Global
   global gb_current_page_changed

   # Check for project
   if ![MH_IsCurrentProject] {return}

   if {$gb_current_page_changed == 1} {
      if ![Question_Dialog . "Page has been modified.  Do you wish
to save the page before displaying?" \
         "Save" "Use old"] {
         MH_SaveHelpPage
      }
   }

   # Now show the run-time help
   MH_MiniHelp [.name_fr.name get]
}

###############################################################
# The procedure for finding tags in the help
proc MH_CreateHelp { name } {

   # Globals
   global gt_edit_help search FONT_BOLD FONT_ITALIC \
      FIXED_FONT TEXT_COLOUR gs_sgml_structure
   global $gs_sgml_structure

   # Format list
   set format_list [list TITLE B I TT UL CENTER]

   # Keyworks
   set TITLE "TITLE"
   set B  "B"
   set I "I"
   set TT "TT"
   set UL "UL"
   set CENTER "CENTER"
   set LEFT "LEFT"
   set RIGHT "RIGHT"

   # During save of the images, save the image tags so that
   # we don't pick them up during parsing for hyperlinks.
   set tags [$gt_edit_help tag names]
   lappend whole_list sel

   # Now for any embedded images
   set image_list [$gt_edit_help window names]
   foreach image $image_list {

      # Get the location of the displayed image
      set image_name [$image.name cget -text]
      set tag "<IMG SRC=\"$image_name\">"
      set window_num [string range $image [expr [string last . \
         $image] +1] end]
      # Info_Dialog . "$image | $tag | $window_num"

      # Get the location of the displayed image
      lappend whole_list image$window_num
      set location [$gt_edit_help tag nextrange image$window_num 1.0]
      if {$location != ""} {
         # Insert the tag (just after the image - this ensures
         # that other formatting instructions are picked up 
         # correctly.
         destroy $image
         $gt_edit_help tag delete image$window_num
         $gt_edit_help insert [lindex $location 0] $tag
      } else {
         Info_Dialog . "Unable to find embedded image location!"
      }
   }

   # Loop round the declared types
   foreach type $format_list {
      lappend whole_list $type
      # Note that location here is a two element list
      set location [$gt_edit_help tag nextrange $type 1.0]
      set format [eval subst \$$type]
      # Info_Dialog . $location
      while {$location != ""} {
         # Replace the tagged area with the formatting
         # instructions
         $gt_edit_help tag remove $type [lindex $location 0] \
            [lindex $location 1]
         $gt_edit_help insert [lindex $location 1] "</$format>"
         $gt_edit_help insert [lindex $location 0] "<$format>"
         set location [$gt_edit_help tag nextrange $type 1.0]
      }
   }

   # Now save the hyperlinks - get a tag list and extract
   # all non-hyperlink tags.
   foreach format $whole_list {
      while {[lsearch -exact $tags $format] != -1} {
         set position [lsearch -exact $tags $format]
         set tags [lreplace $tags $position $position]
      }
   }
   #Info_Dialog . $tags
   foreach type $tags {
      # For each range in question, tag the selection
      set location [$gt_edit_help tag nextrange $type 1.0]
      # Info_Dialog . $location
      while {$location != ""} {
         # Replace the tagged area with the formatting
         # instructions
         $gt_edit_help tag remove $type [lindex $location 0] \
            [lindex $location 1]
         $gt_edit_help insert [lindex $location 1] "</XREF>"
         $gt_edit_help insert [lindex $location 0] "<XREF PAGE=\"$type\">"
         set location [$gt_edit_help tag nextrange $type 1.0]
      }
   }

   # Finally convert all newlines and paragraphs to tags
   # For each range in question, tag the selection
   set location [$gt_edit_help search "\n" 1.0 "end -1c"]
   while {$location != ""} {
      # Replace the tagged area with the formatting
      # instructions
      if {[$gt_edit_help get "$location +1c"] == "\n"} {
         $gt_edit_help delete $location "$location +2c"
         $gt_edit_help insert $location "<P>"
      } else {
         $gt_edit_help delete $location "$location +1c"
         $gt_edit_help insert $location "<BR>"
      }
      set location [$gt_edit_help search "\n" 1.0 "end -1c"]
   }
   set location [$gt_edit_help search "<P>" 1.0 "end -1c"]
   while {$location != ""} {
      # Replace the tagged area with the formatting
      # instructions
      $gt_edit_help insert $location "\n\n"
      set location [$gt_edit_help search "<P>" "$location +5c" "end -1c"]
   }
   set location [$gt_edit_help search "<BR>" 1.0 "end -1c"]
   while {$location != ""} {
      # Replace the tagged area with the formatting
      # instructions
      $gt_edit_help insert "$location +4c" "\n"
      set location [$gt_edit_help search "<BR>" "$location +4c" "end -1c"]
   }

   # Insert the page header and footer
   set lvl [subst \$${gs_sgml_structure}(${name}_Level)]
   $gt_edit_help insert 1.0 "<PAGE ID=\"$name\" LVL=\"$lvl\">\n\n"
   $gt_edit_help insert end "\n\n</PAGE>"

   # Disable text by default
   # $gt_edit_help configure -state disabled
}

###############################################################
# The procedure for finding tags in the help
proc MH_SaveHelpPage { } {

   # Globals
   global gt_edit_help gb_current_page_changed \
      gb_show_formatted gs_image_base_dir \
      gs_sgml_structure gb_keep_old_text
   global $gs_sgml_structure

   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Check for a valid name
   set name [.name_fr.name get]
   if {$name == ""} {
      set gb_keep_old_text 1
      MH_NewHelpPage
      update
      tkwait window .new_page
   }

   if {$gb_show_formatted == 1} {
      # Create the tagged text
      MH_CreateHelp $name
   }

   # Perform substitution of invalid tcl characters
   # e.g. ${}[]
   set text [$gt_edit_help get 1.0 end]
   set charlist [list \$ \{ \} \[ \] \\ ]
   set replacelist [list \\\&dollar\; \\\&lbrace\; \\\&rbrace\; \\\&lsquare\; \
      \\\&rsquare\; \\\&backslash\; ]
   set count 0
   set char [lindex $charlist $count]
   set repl [lindex $replacelist $count]
   while {$count < [llength $charlist]} {
      # Search and replace
      # Info_Dialog . "Char: $char   Repl: $repl"
      regsub -all \\$char $text $repl text
      # Info_Dialog . $text

      # Get the next item
      incr count
      set char [lindex $charlist $count]
      set repl [lindex $replacelist $count]
   }

   # Save the resulting text to the structure
   set ${gs_sgml_structure}(${name}_Text) $text

   # If show is true, redisplay the text in formatted form.
   # Otherwise, leave unformatted.
   if {$gb_show_formatted == 1} {
      # Now redisplay the text and disable the save button
      MH_MarkupToText $gt_edit_help INSERT 0 $gs_image_base_dir \
         [$gt_edit_help get 1.0 end]
   }

   # Reset status
   .name_fr.save configure -state disabled
   set gb_current_page_changed 0
}

###############################################################
# The procedure for finding tags in the help
proc MH_DeleteHelpPage { name } {

   # Globals
   global gas_pages gt_edit_help gb_current_page_changed \
      gb_project_changed gas_history
   set page "gs_HELPTEXT_$name"
   global $page
   
   # Check for project
   if ![MH_IsCurrentProject] {return}

   # Ask the user for confirmation
   if ![Question_Dialog . "Do you really want to delete page '$name'?" \
      "Delete" "Cancel"] {
      # Perform deletion.
      unset $page
      set item [lsearch -exact $gas_pages $page]
      set gas_pages [lreplace $gas_pages $item $item]
      set gas_history [lreplace $gas_history $item $item]

      # And blank out the edit areas of the screen
      $gt_edit_help delete 1.0 end
      MH_DisableEdit
      set gb_current_page_changed 0
      set gb_project_changed 1

      # Display the page title in the entry
      .name_fr.name configure -state normal
      .name_fr.name delete 0 end
      .name_fr.name configure -state disabled
      .name_fr.save configure -state disabled
   }
}

###############################################################
# The procedure to copy text to the clipboard
proc MH_CopyText { delete } {

   # Global
   global gt_edit_help

   # Copy the selected text into the cut buffer, then
   # delete if the flag 'delete' is set.
   if [catch {$gt_edit_help get sel.first}] {return}
   clipboard clear
   clipboard append [$gt_edit_help get sel.first sel.last]

   # And delete/allow a save if data deleted
   if {$delete == 1} { 
      $gt_edit_help delete sel.first sel.last
      set gb_project_changed      1
      set gb_current_page_changed 1
      .name_fr.save configure -state normal
   }
}

###############################################################
# The procedure to goto a specified page from the history
proc MH_PasteText { } {

   # Global
   global gt_edit_help

   # Perform paste if data exists in the cut buffer
   if ![catch {selection get -selection CLIPBOARD} text] {
      if {$text != ""} {
         $gt_edit_help insert [$gt_edit_help index insert] $text
      }
   }
}

###############################################################
# The procedure to add a tag to the current selection
proc MH_AddTag { type } {

   # Globals
   global gt_edit_help search FONT_BOLD FONT_ITALIC \
      FIXED_FONT TEXT_COLOUR RED GREEN gb_project_changed \
      gb_current_page_changed

   # Format list
   set format_list [list TITLE B I TT UL CENTER LEFT RIGHT]

   # If the input argument is not in this list then it is 
   # not a valid type.
   if {[lsearch -exact $format_list $type] == -1} {
      Info_Dialog . "Invalid/unknown tag type specified!"
      return
   }

   # Keyworks and formatting
   set TITLE "TITLE"
   set TITLE_FORMAT "-font $FONT_BOLD -underline True -justify center"
   set BOLD  "B"
   set B_FORMAT "-font $FONT_BOLD"
   set ITALIC "I"
   set I_FORMAT "-font $FONT_ITALIC"
   set FIXED "TT"
   set TT_FORMAT "-font $FIXED_FONT"
   set UNDERLINE "UL"
   set UL_FORMAT "-underline True"
   set CENTER "CENTER"
   set CENTER_FORMAT "-justify center"
   set LEFT "LEFT"
   set LEFT_FORMAT "-justify left"
   set RIGHT "RIGHT"
   set RIGHT_FORMAT "-justify right"

   # Get the selected text, and format with the 
   # characteristic given, flagging the block with
   # the tag.  Check for a selection first.
   if [catch {$gt_edit_help get sel.first}] {return}

   # Certain tags preclude other tags (e.g. justification)
   if {$type == "CENTER" || $type == "LEFT" || $type == "RIGHT"} {
      MH_RemoveTags JUSTIFY
   }

   # Add the new tag type
   $gt_edit_help tag add $type sel.first sel.last
   set format [eval subst \$${type}_FORMAT]
   eval $gt_edit_help tag configure $type $format
   $gt_edit_help tag raise $type

   # And allow a save
   set gb_project_changed      1
   set gb_current_page_changed 1
   .name_fr.save configure -state normal
}

###############################################################
# The procedure to remove all tags from the current selection
proc MH_RemoveTags { type } {

   # Global
   global gt_edit_help FIXED_FONT gb_project_changed \
      gb_current_page_changed

   # Allowable values for type are:
   # XREF - Hyperlink references
   # CHARACTER - Character/font formatting
   # JUSTIFY - Justification
   # ALL - All formatting

   set as_character [list BOLD UNDERLINE TITLE ITALIC FIXED]
   set as_justify [list CENTER LEFT RIGHT]
   set as_non_xref [list BOLD UNDERLINE TITLE ITALIC FIXED \
      CENTER LEFT RIGHT sel]
   set delete_xref 0
   set al_del_list ""

   # Determine what needs to be done
   switch -exact $type {
      XREF {set delete_xref 1}
      CHARACTER {set al_del_list $as_character}
      JUSTIFY {set al_del_list $as_justify}
      ALL {set al_del_list [list [join $as_character $as_justify]]
         set delete_xref 1}
      default {
         Info_Dialog . "Error: Unrecognised clear command"
         return
      }
   }

   # Get the selected text, and unformat
   if [catch {$gt_edit_help get sel.first}] {return}
   set this [$gt_edit_help index sel.first]
   set last [$gt_edit_help index sel.last]
   while {$this != $last} {

      # Delete selected formatting commands
      foreach tag $al_del_list {
         #Info_Dialog . "Removing $tag from position $this"
         $gt_edit_help tag remove $tag $this
      }
      # If delete_xref selected find the xrefs and delete
      if {$delete_xref == 1} {
         set tags [$gt_edit_help tag names $this]
         foreach tag $as_non_xref {
            set loc [lsearch -exact $tags $tag]
            if {$loc != -1} {set tags [lreplace $tags $loc $loc]}
         }
         foreach xref $tags {
            #Info_Dialog . "Removing xref $tag from position $this"
            $gt_edit_help tag remove $xref $this
         }
      }

      # Increment to the next character
      set this [$gt_edit_help index "$this + 1 chars"]
   }

   # And allow a save
   set gb_project_changed      1
   set gb_current_page_changed 1
   .name_fr.save configure -state normal
}

###############################################################
# Works out whether a project is currently displayed.
# If one is not 0 is returned and the bell is rung.
proc MH_IsCurrentProject {} {
   
   global gs_current_project
   if {$gs_current_project == ""} {
      bell
      return 0
   } 
   return 1
}

###############################################################
# The procedure to disable the edit field
proc MH_DisableEdit {} {
   
   global gt_edit_help DEFAULT_COLOUR
   $gt_edit_help configure -state disabled -background $DEFAULT_COLOUR
   .name_fr.preview configure -state disabled
   .name_fr.open configure -state disabled
}

###############################################################
# The procedure to disable the edit field
proc MH_EnableEdit {} {
   
   global gt_edit_help TEXT_COLOUR
   $gt_edit_help configure -state normal -background $TEXT_COLOUR
   .name_fr.preview configure -state normal
   .name_fr.open configure -state normal
}

###############################################################
# Show the main interface
###############################################################

# Source other files
MH_SourceOther global.tcl
MH_SourceOther dialog.tcl
MH_SourceOther help.tcl
MH_SourceOther parser.tcl
MH_SourceOther utils.tcl
MH_SourceOther minihelp_filter.tcl

###############################################################
# Globals for this module
set g_selected_file ""
set g_selected_file_filter ""

# Flags
set gas_pages ""
set gas_history ""
set gb_current_page_changed 0
set gb_project_changed      0
set gs_current_project      ""
set gs_new_project_name     ""
set gs_MH_NewHelpPage        ""
set gb_show_formatted       1
set gb_keep_old_text        0
set gb_page_autocommit      1
set gs_image_base_dir       ""
set gs_indent_string        "     "

###############################################################
# Set application defaults

# Fonts
option add *font $FONT_NORMAL startupFile

# Highlight
option add *highlightThickness 0

# Colours
option add *background  $DEFAULT_COLOUR startupFile
option add *Text.background  $TEXT_COLOUR startupFile
option add *Entry.background  $TEXT_COLOUR startupFile

###########################################################################
# Create the mini-help screen
wm title . "Edit Help - <no project>"
wm protocol . WM_DELETE_WINDOW "MH_Exit"

###########################################################################
# Create Menu Bar
frame .mbar -relief raised -bd 2
pack .mbar -side top -fill x

# Create the buttons   
menubutton .mbar.file -text File -underline 0 -menu .mbar.file.menu
menubutton .mbar.edit -text Edit -underline 0 -menu .mbar.edit.menu
menubutton .mbar.format -text Format -underline 0 -menu .mbar.format.menu
menubutton .mbar.insert -text Insert -underline 0 -menu .mbar.insert.menu
menubutton .mbar.page -text Page -underline 0 -menu .mbar.page.menu
menubutton .mbar.navigate -text Navigate -underline 0 -menu .mbar.navigate.menu
menubutton .mbar.windows -text Windows -underline 0 -menu .mbar.windows.menu
pack .mbar.file .mbar.edit .mbar.format .mbar.insert .mbar.page \
   .mbar.navigate .mbar.windows -side left

# Create each menu item
menu .mbar.file.menu -tearoff 0
   .mbar.file.menu add command -label "New" -command {
      set gs_current_project [File_Select "Set New Project File"]
      if {$gs_current_project != ""} {
         MH_MarkupToInternal $gt_edit_help $gs_current_project
         MH_AddTitle
         .mbar.windows.menu add command -label \
            $gs_current_project -command \
            "MH_SwitchToProject $gs_sgml_structure"
      }
   } -underline 0
   .mbar.file.menu add command -label "Open" -command \
      "MH_OpenProject" -underline 0 -accelerator "Ctrl-O"
   .mbar.file.menu add command -label "Close" -command \
      "MH_CloseProject" -underline 0
   .mbar.file.menu add separator
   .mbar.file.menu add command -label "Change Title..." -command \
      "MH_AddTitle" -underline 7
   .mbar.file.menu add command -label "Change Base Dir..." -command \
      {MH_ImageChangeBase} -underline 7
   .mbar.file.menu add separator
   .mbar.file.menu add command -label "Save" -command \
      {MH_SaveProject $gs_current_project} -underline 0 \
      -accelerator "Ctrl-S"
   .mbar.file.menu add command -label "Save As..." -command "MH_SaveProjectAs" -underline 0
   .mbar.file.menu add separator
   .mbar.file.menu add command -label "Exit" -command "MH_Exit" -underline 0
menu .mbar.edit.menu -tearoff 0
   .mbar.edit.menu add separator
   .mbar.edit.menu add command -label "Cut" -command \
      "MH_CopyText 1" -underline 2 -accelerator "Ctrl-X"
   .mbar.edit.menu add command -label "Copy" -command \
      "MH_CopyText 0" -underline 0 -accelerator "Ctrl-C"
   .mbar.edit.menu add command -label "Paste" -command \
      "MH_PasteText" -underline 0 -accelerator "Ctrl-V"
   .mbar.edit.menu add separator
   .mbar.edit.menu add command -label "Remove Link" -command \
      "MH_RemoveTags XREF" -underline 0
   .mbar.edit.menu add command -label "Remove Formatting" -command \
      "MH_RemoveTags CHARACTER" -underline 0
   .mbar.edit.menu add command -label "Remove Justification" -command \
      "MH_RemoveTags JUSTIFY" -underline 0
   .mbar.edit.menu add command -label "Remove All" -command \
      "MH_RemoveTags ALL" -underline 0
menu .mbar.format.menu -tearoff 0
   .mbar.format.menu add command -label Bold -command "MH_AddTag B" \
      -underline 0 -accelerator "Ctrl-B"
   .mbar.format.menu add command -label Italic -command \
      "MH_AddTag I" -underline 0 -accelerator "Ctrl-I"
   .mbar.format.menu add command -label Underline -command \
      "MH_AddTag UL" -underline 0 -accelerator "Ctrl-U"
   .mbar.format.menu add command -label Serif -command \
      "MH_AddTag TT" -underline 0 -accelerator "Ctrl-T"
   .mbar.format.menu add separator
   .mbar.format.menu add command -label Centre -command \
      "MH_AddTag CENTER" -underline 0
   .mbar.format.menu add command -label Left -command \
      "MH_AddTag LEFT" -underline 0
   .mbar.format.menu add command -label Right -command \
      "MH_AddTag RIGHT" -underline 0
   .mbar.format.menu add separator
   .mbar.format.menu add command -label Title -command \
      "MH_AddTag TITLE" -underline 0
   .mbar.format.menu add command -label Hyperlink... -command \
      {MH_BindLink "" "" ""} -underline 0 -accelerator "Ctrl-L"
menu .mbar.page.menu -tearoff 0
   .mbar.page.menu add command -label "New..." -command {
      if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
         MH_SaveHelpPage
      }
      if {$gb_current_page_changed == 0} {
         MH_NewHelpPage 
      } else {
         if ![Question_Dialog . "Help text on this page has changed!" \
            "Discard Changes" "Cancel"] {
            MH_NewHelpPage 
         }
      }
   } -underline 0
   .mbar.page.menu add command -label "Rename..." -command \
      {MH_RenamePage} -underline 0
   .mbar.page.menu add command -label Delete -command {
      set name [.name_fr.name get]
      if {$name != ""} {
         MH_DeleteHelpPage $name
      }
   } -underline 0
   .mbar.page.menu add separator
   .mbar.page.menu add command -label "Re-Parse" -command {
      if {$gb_show_formatted == 1} {
         MH_MarkupToText $gt_edit_help REFORMAT 0 $gs_image_base_dir
      } else {bell}
   }
   .mbar.page.menu add checkbutton -label "Display Formatted" -command {
      if {$gb_show_formatted == 1} {
         MH_MarkupToText $gt_edit_help INSERT 0 $gs_image_base_dir \
            [$gt_edit_help get 1.0 end]
      } else {
         MH_CreateHelp [.name_fr.name get]
      }
   } -variable gb_show_formatted
   .mbar.page.menu add checkbutton -label "Autocommit Pages" \
      -variable gb_page_autocommit
menu .mbar.navigate.menu -tearoff 0
   .mbar.navigate.menu add command -label "Previous Page" -command {
      if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
         MH_SaveHelpPage
      }
      if {$gb_current_page_changed == 0} {
         MH_EditNavigateBack
      } else {
         if ![Question_Dialog . "Help text on this page has changed!" \
            "Discard Changes" "Cancel"] {
            MH_EditNavigateBack
         }
      }
   } -underline 0 -accelerator "Ctrl-P"
   .mbar.navigate.menu add command -label "History ..." -command {
      if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
         MH_SaveHelpPage
      }
      if {$gb_current_page_changed == 0} {
         MH_SelectEditHelpPage . HISTORY
      } else {
         if ![Question_Dialog . "Help text on this page has changed!" \
            "Discard Changes" "Cancel"] {
            MH_SelectEditHelpPage . HISTORY
         }
      }
   } -underline 0 -accelerator "Ctrl-H"
   .mbar.navigate.menu add command -label "Select Page ..." -command {
      if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
         MH_SaveHelpPage
      }
      if {$gb_current_page_changed == 0} {
         MH_SelectEditHelpPage . FULL
      } else {
         if ![Question_Dialog . "Help text on this page has changed!" \
            "Discard Changes" "Cancel"] {
            MH_SelectEditHelpPage . FULL
         }
      }
   } -underline 0 -accelerator "Ctrl-G"

menu .mbar.insert.menu -tearoff 0
   .mbar.insert.menu add command -label "Image..." -command \
      {if ![MH_IsCurrentProject] {return}; \
      set gb_current_page_changed 1; .name_fr.save configure \
      -state normal; MH_InsertImage $gt_edit_help insert} -underline 0 \
      -accelerator "Ctrl-I"
   .mbar.insert.menu add command -label "From File..." -command \
      "MH_ImportFile" -underline 5
   .mbar.insert.menu add command -label "HTML..." -command \
      "MH_ImportHTML" -underline 1

menu .mbar.windows.menu -tearoff 0

###############################################################
# Create the page label
set frame [frame .name_fr -relief groove -borderwidth $RIDGE_BORDER]
label $frame.label -text "Page Label"
entry $frame.name -width 25 -state disabled
button $frame.save -text Save -command {MH_SaveHelpPage}
button $frame.preview -text Preview... -command {MH_PreviewPage}
$frame.save configure -state disabled -disabledforeground ""
button $frame.open -text "..." -command {
   if {$gb_current_page_changed == 1 && $gb_page_autocommit == 1} {
      MH_SaveHelpPage
   }
   if {$gb_current_page_changed == 0} {
      MH_SelectEditHelpPage . FULL
   } else {
      if ![Question_Dialog . "Help text on this page has changed!" \
         "Discard Changes" "Cancel"] {
         MH_SelectEditHelpPage . FULL
      }
   }
}
pack $frame.label $frame.name $frame.open $frame.save \
   $frame.preview -side left -fill x -padx $DEFAULT_PADDING \
   -pady $DEFAULT_PADDING
pack $frame -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING -fill x

###############################################################
# Create the text widget
frame .fr -borderwidth $RIDGE_BORDER -relief groove
pack .fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING -side top \
   -expand true -fill both
set gt_edit_help [ScrolledText .fr.help 70 30 0]
$gt_edit_help configure -font $FONT_NORMAL -bg $TEXT_COLOUR 
$gt_edit_help tag bind sel <3> {+MH_PopupMenu $gt_edit_help .mbar.format.menu}
$gt_edit_help tag bind sel <Control-3> {+MH_PopupMenu $gt_edit_help .mbar.edit.menu}
$gt_edit_help tag bind sel <Shift-3> {+MH_PopupMenu $gt_edit_help .mbar.justify.menu}
bind $gt_edit_help <KeyPress> {
   set gb_current_page_changed 1
   set ${gs_sgml_structure}(Modified) 1
   $frame.save configure -state normal
}
pack .fr.help -side top -anchor w

# Initially disable the field
MH_DisableEdit

# Center on the screen
Centre_Dialog . 

# Pop-up the open dialogue
set gs_current_project [lindex $argv 0]
if {$gs_current_project != ""} {
   MH_ReadProject
}

