###############################################################
# DISPLAY MODULE FOR THE SGML TEXT
#
# Charlie KEMPSON - charlie@siren.demon.co.uk
# http://public.logica.com/~kempsonc/tknet.htm
###############################################################

###############################################################
#
#    Copyright (c) 1995-1996 Charlie Kempson
#
#    This program is free software; you can redistribute it 
#    and/or modify it under the terms of the GNU General 
#    Public License as published by the Free Software 
#    Foundation (version 2 of the License).
#
#    This program is distributed in the hope that it will 
#    be useful, but WITHOUT ANY WARRANTY; without even the 
#    implied warranty of MERCHANTABILITY or FITNESS FOR A 
#    PARTICULAR PURPOSE.  See the GNU General Public License 
#    for more details.
#
#    For a copy of the GNU General Public License, write to the 
#    Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
#    MA 02139, USA.
###############################################################

###############################################################
# Globals for this module
set gb_show_formatted 1

# Flags
set gas_history ""

# Geometry
set TKNET_HELP_GEOMETRY "+200+200"

###############################################################
# Show the mini-help panel
proc MH_MiniHelp {text} {

   # The argument to MH_MiniHelp must be the name of the page
   # as created by MiniHelp.  Th global avariable containing
   # the help text has this name plus a gs_HELPTEXT_ prefix
   # for uniqueness.

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING FONT_NORMAL \
      gt_mini_help TKNET_HELP_GEOMETRY gas_history \
      gs_image_base_dir gs_sgml_structure 
   global $gs_sgml_structure 

   if [catch {set local_text [subst \$${gs_sgml_structure}(${text}_Text)]} \
      error] {
      Info_Dialog . "Unable to read page $text: $error"
      return 
   }

   WatchCursor
   # Test for window
   if ![winfo exists .mini_help_window] {
   
      # Create the mini-help screen
      set window [toplevel .mini_help_window]
      wm title .mini_help_window "Help"
      wm transient .mini_help_window .
      wm geometry .mini_help_window $TKNET_HELP_GEOMETRY

      ###########################################################################
      # Create Menu Bar
      frame $window.mbar -relief raised -bd 2
      pack $window.mbar -side top -fill x

      # Create the buttons   
      menubutton $window.mbar.file -text File -underline 0 -menu \
         $window.mbar.file.menu
      menubutton $window.mbar.navigate -text Navigate -underline 0 \
         -menu $window.mbar.navigate.menu
      pack $window.mbar.file $window.mbar.navigate \
         -side left

      # Create each menu item
      menu $window.mbar.file.menu -tearoff 0
         $window.mbar.file.menu add command -label "Close" \
            -command "destroy $window" -underline 0
      menu $window.mbar.navigate.menu -tearoff 0
         $window.mbar.navigate.menu add command -label "Back" \
            -command MH_NavigateBack -underline 0
         $window.mbar.navigate.menu add separator
         $window.mbar.navigate.menu add command -label "Contents" \
            -command "MH_MiniHelp Contents" -underline 0
         $window.mbar.navigate.menu add command -label "Browse..." \
            -command "MH_SelectHelpPage $window FULL" -underline 0
         $window.mbar.navigate.menu add command -label "History..." \
            -command "MH_SelectHelpPage $window HISTORY" -underline 0

      # Create the menu
      tk_menuBar $window.mbar $window.mbar.file \
      $window.mbar.navigate

      ###############################################################
      # Create the log panel
      frame .mini_help_window.fr -borderwidth $RIDGE_BORDER \
         -relief groove
      pack .mini_help_window.fr -padx $DEFAULT_PADDING -pady \
         $DEFAULT_PADDING -side top -expand true -fill both
      set gt_mini_help [ScrolledText \
         .mini_help_window.fr.help 80 25 0]
      $gt_mini_help configure -font $FONT_NORMAL
      pack .mini_help_window.fr.help -side top -anchor w
   
      ###############################################################
      # Create the buttons below the frame
      frame .mini_help_window.button_frame -borderwidth $DEFAULT_PADDING
      pack .mini_help_window.button_frame -side bottom -fill x
      button .mini_help_window.button_frame.close -font $FONT_NORMAL \
          -text Close -command { destroy .mini_help_window }
      pack .mini_help_window.button_frame.close

   }

   # Check for minimised state and raise it
   wm deiconify .mini_help_window
   raise .mini_help_window
   
   # Put the help text in the window
   $gt_mini_help configure -state normal
   MH_MarkupToText $gt_mini_help INSERT 1 $gs_image_base_dir $local_text 
   $gt_mini_help configure -state disabled -font $FONT_NORMAL

   # Add to the history list (removing any other references to
   # the page on the way - there will be at most one given the
   # nature of the check)
   set item [lsearch -exact $gas_history $text]
   if {$item != -1} {set gas_history [lreplace $gas_history $item $item]}
   lappend gas_history $text

   NormalCursor
}

###############################################################
# The procedure for placing an image into the text
proc MH_InsertImage { widget location {image_name ""} } {

   # Globals
   global gs_image_base_dir RIDGE_BORDER DEFAULT_PADDING \
      TEXT_COLOUR

   # Images are created for the current page with the 
   # names 1,2,3 etc... and are tagged with the tag
   # name 'image<N>' where N is the image name.

   # Extract the image name - text is in the format
   # "IMG SRC=filename", also understand "IMG=filename"
   # Note that the image may be specified using an 
   # absolute path, or may use a predefined image 
   # path.  Check the image for validity.
   set image_path $gs_image_base_dir
   if {$image_name == ""} {
      set image_name [File_Select "Select Image..." "*" \
         $gs_image_base_dir]
   }
   if {$image_name == ""} {return}

   # If an absolute path is specified, check for existence.
   # If not found then try the base image path, if specified.
   set b_found 0
   if {[file dirname $image_name] != "."} {
      if [file exists $image_name] {set b_found 1}
   }
   if {$b_found == 0} {
      # Strip all path information from image
      set pos [string last "/" $image_name]
      if {$pos > 0} {set image_name [string range $image_name \
         [expr $pos + 1] end]}
      set pos 0
      while {$b_found == 0 && $pos < [llength $image_path]} {
         set file [lindex $image_path $pos]
         # Info_Dialog . "$file/$image_name"
         if [file exists "$file/$image_name"] {
            # Found the image
            set image_name "$file/$image_name"
            set b_found 1
         }
         incr pos
      }
   }

   # Check for existence
   # Info_Dialog . $image_name
   if [file exists $image_name] {
      set image [Create_Image $image_name]
   } else {
      # Image does not exist
      set image -1
   }

   if {$image != -1} {
      set num_windows [llength [$widget window names]]
      incr num_windows

      # Create the image in a frame
      frame $widget.$num_windows
      label $widget.$num_windows.l -image $image -background \
         $TEXT_COLOUR -bd 0 -highlightthickness 0
      label $widget.$num_windows.name -text $image_name
      pack $widget.$num_windows.l -side top
      $widget window create $location -padx 2 -pady 2 \
         -window $widget.$num_windows
      $widget tag add image$num_windows $location
   } else {
      set num_windows [llength [$widget window names]]
      incr num_windows

      # Create the image in a frame
      frame $widget.$num_windows
      label $widget.$num_windows.name -text $image_name
      pack $widget.$num_windows.name -side top
      $widget window create $location -padx 2 -pady 2 \
         -window $widget.$num_windows
      $widget tag add image$num_windows $location
   }
}

###############################################################
# The procedure to navigate backwards through the history
proc MH_NavigateBack {} {

   # Globals
   global gas_history 

   # The list gas_history contains a list of all help pages
   # visited.  Simply extract the second to last page, and
   # delete the last item in the list, then dislpay the 
   # page.
   if {[llength $gas_history] < 2} {
      # Nowhere to return to
      return
   }
   
   # Get the last page
   set help [lindex $gas_history [expr [llength $gas_history] - 2]]

   # Delete the two last items (note that one of them is recreated
   # when we call MH_MiniHelp).
   set gas_history [lreplace $gas_history [expr [llength $gas_history] \
      - 2] end]

   # And show the help page
   MH_MiniHelp $help
}

###############################################################
# The procedure to let you select a page
proc MH_SelectHelpPage { parent type } {

   # Type may be HISTORY or FULL

   # Globals
   global RIDGE_BORDER DEFAULT_PADDING TEXT_COLOUR \
      gas_history gs_sgml_structure
   global $gs_sgml_structure

   # Test for window
   # Popup a selection window
   set window .select_help
   if [winfo exists $window] {
       # Pop it up!
       wm deiconify $window
       raise $window
       update
       return
   }

   toplevel $window
   if {$type == "HISTORY"} {
      wm title $window "History"
      set as_list $gas_history
	} else {
		wm title $window "Select Page"
		set as_list [subst \$${gs_sgml_structure}(TOC)]
	}
	wm transient $window .

	# Create a frame containing a list
	frame $window.sel_fr -borderwidth $RIDGE_BORDER -relief groove
	pack $window.sel_fr -padx $DEFAULT_PADDING -pady $DEFAULT_PADDING \
		-side top -fill x
	set list [ScrolledList $window.list_fr 0 20 browse 0]
	$list configure -bg $TEXT_COLOUR
	pack forget $window.list_fr
	pack $window.list_fr -side top -fill both -padx $DEFAULT_PADDING \
		-pady $DEFAULT_PADDING -expand true

   # Add the help pages title to the list
   foreach item $as_list {
      # Get the help text title for the page
      set text "Unknown Help Page"
      set local_text [subst \$${gs_sgml_structure}(${item}_Text)]
      set start [string first "<title>" $local_text]
      if {$start == -1} {set start [string first "<TITLE>" $local_text]}
      set end [string first "</title>" $local_text]
      if {$end == -1} {set end [string first "</TITLE>" $local_text]}

      if {$start != -1} {
         set start [expr [string wordend $local_text [expr $start + 1]] +1]
         set end [expr [string wordstart $local_text $end] -1]
         set text [string range $local_text $start $end]
         set text [string trimleft $text]
      } else {
         Info_Dialog . "Error in help text (${item})!  
All pages must begin with a title e.g. 
<title>Contents</title> or
<TITLE>Contents</TITLE>"
      }

      # Add indentation for the levels
      # Info_Dialog . "$text : [subst \$${gs_sgml_structure}(${item}_Level)]"
      set litem ""
      for {set count 1} {$count < [subst \
         \$${gs_sgml_structure}(${item}_Level)]} {incr count} {
         append litem "......."
      }
      append litem $text
      $list insert end $litem
   }

	# Now for a goto, close and contents button
	set frame [frame $window.button_fr]
	pack $frame -side bottom -fill x
	button $frame.close -text Close -command "destroy $window"
	button $frame.new -text New -command "destroy $window; MH_NewHelpPage"
	button $frame.goto -text Goto -command "MH_GotoHelp $window $list $type"
	bind $list <Double-ButtonRelease-1> "MH_GotoHelp $window $list $type"
	pack $frame.close $frame.new $frame.goto -side right \
		-padx $DEFAULT_PADDING -pady $DEFAULT_PADDING

	# And now centre it on its parent
	Centre_Dialog $window widget $parent
}

###############################################################
# The procedure to goto a specified page from the history
proc MH_GotoHelp { window list type } {

   # Globals
   global gas_history gs_sgml_structure

   if {$type == "HISTORY"} {
      set as_list $gas_history
	} else {
      global $gs_sgml_structure
		set as_list [subst \$${gs_sgml_structure}(TOC)]
	}

   set selected [$list curselection]
   if {$selected == ""} {return}
   set text [lindex $as_list $selected]

   if {$type == "HISTORY"} {
      set gas_history [lreplace $gas_history $selected end]
   }

   # Show the new page
   MH_MiniHelp $text
   after 400 "destroy $window"
}

###############################################################
# The procedure to start up an edit session using MiniHelp
proc MH_EditHelp { } {

   # Globals
   global TKNET_USER GLOBAL_INCLUDE

   # Check that the user is root - if not, disallow operation
   if {$TKNET_USER != "root"} {
      Info_Dialog . "You must be root to edit the help!"
   } else {
      # Start the edit session
      if [catch {eval exec wish -f $GLOBAL_INCLUDE/modules/MiniHelp.tcl &} errortext] {
         Info_Dialog . "MiniHelp edit session failed with error:\n$errortext"
      }
   }
}
